#!/bin/bash

usage()
{
cat <<-USAGE
usage: nodes.sh [OPTIONS] LOGDIR

options:
    -h      show this help text
USAGE
}

REGEX_NODES='\(Transition\|Rate\) matrix: .* nodes'

main()
{
    local opt
    while getopts ":h" opt; do
        case $opt in
            h)
                usage
                exit
                ;;
            \?)
                echo "Invalid option: -$OPTARG" >&2
                exit 1
                ;;
            :)
                echo "Option -$OPTARG requires an argument" >&2
                exit 1
                ;;
        esac
    done
    shift $((OPTIND-1))

    if [[ $# != 1 ]]; then
        usage
        exit 1
    fi

    logdir=$1
    engine='mtbdd'
    shift

    read_dist_settings

    for prop in $logdir/*; do
        if [[ -d $prop ]]; then
            p=$(basename $prop)
            break
        fi
    done

    resultdir=results
    mkdir -p $resultdir

    all_in_one
    if [[ -z $noredundant ]]; then
        one_by_one
    else
        one_by_one_noredundant
    fi
}

read_dist_settings()
{
    from=$(< $logdir/from)
    to=$(< $logdir/to)
    if [[ -e $logdir/noredundant ]]; then noredundant=1; fi
}

all_in_one()
{
    local datfile=$resultdir/nodes_all_in_one.dat
    echo "# size nodes" > $datfile
    for i in $(seq $from $to); do
        f=$logdir/$p/$engine/all_in_one/${i}.log
        if is_incomplete $f; then break; fi

        nodes=$(grep "$REGEX_NODES" $f | cut -f 3 -d' ')
        echo $i $nodes >> $datfile
    done
}

one_by_one()
{
    local datfile=$resultdir/nodes_one_by_one.dat
    echo "# size nodes max" > $datfile
    for i in $(seq $from $to); do
        local dir=$logdir/$p/$engine/one_by_one/$i
        if is_incomplete $dir; then break; fi

        grep "$REGEX_NODES" $dir/*.log --no-filename | cut -f 3 -d' ' > nodes
        sum=$(paste -s -d+ nodes | bc)
        max=$(awk '$0>x { x=$0 } END { print x }' nodes)
        echo $i $sum $max >> $datfile
    done
    rm nodes
}

one_by_one_noredundant()
{
    local datfile=$resultdir/nodes_one_by_one.dat
    local dir=$logdir/$p/$engine/one_by_one/$to
    echo "# size nodes max" > $datfile
    for i in $(seq $from $to); do
        local n=$((i - from + 1))
        if is_incomplete $dir/$(ls -1v $dir | head -n $n | tail -1); then break; fi

        ls -1v $dir/*.log | head -n $n | xargs grep "$REGEX_NODES" --no-filename | cut -f 3 -d' ' > nodes
        sum=$(paste -s -d+ nodes | bc)
        max=$(awk '$0>x { x=$0 } END { print x }' nodes)
        echo $i $sum $max >> $datfile
    done
    rm nodes
}

is_incomplete()
{
    grep -r TIMEOUT $1 > /dev/null || grep -r MEMOUT $1 > /dev/null
}

main "$@"

